@@ -57,6 +57,9 @@ db_opts = [
                default='share-snapshot-%s',
                help='Template string to be used to generate share snapshot '
                     'names.'),
+    cfg.StrOpt('share_backup_name_template',
+               default='share-backup-%s',
+               help='Template string to be used to generate backup names.'),
 ]
 
 CONF = cfg.CONF
@@ -311,14 +314,19 @@ def share_instance_create(context, share_id, values):
     return IMPL.share_instance_create(context, share_id, values)
 
 
-def share_instance_delete(context, instance_id, session=None,
+def share_instance_delete(context, instance_id,
                           need_to_update_usages=False):
     """Delete share instance."""
     return IMPL.share_instance_delete(
-        context, instance_id, session=session,
+        context, instance_id,
         need_to_update_usages=need_to_update_usages)
 
 
+def update_share_instance_quota_usages(context, instance_id):
+    """Update share instance quota usages"""
+    return IMPL.update_share_instance_quota_usages(context, instance_id)
+
+
 def share_instance_update(context, instance_id, values, with_share_data=False):
     """Update share instance fields."""
     return IMPL.share_instance_update(context, instance_id, values,
@@ -334,45 +342,47 @@ def share_and_snapshot_instances_status_update(
         current_expected_status=current_expected_status)
 
 
-def share_instances_status_update(context, share_instance_ids, values):
+def share_instance_status_update(context, share_instance_ids, values):
     """Updates the status of a bunch of share instances at once."""
-    return IMPL.share_instances_status_update(
+    return IMPL.share_instance_status_update(
         context, share_instance_ids, values)
 
 
-def share_instances_get_all(context, filters=None):
+def share_instance_get_all(context, filters=None):
     """Returns all share instances."""
-    return IMPL.share_instances_get_all(context, filters=filters)
+    return IMPL.share_instance_get_all(context, filters=filters)
 
 
-def share_instances_get_all_by_share_server(context, share_server_id,
-                                            with_share_data=False):
+def share_instance_get_all_by_share_server(
+    context, share_server_id, with_share_data=False,
+):
     """Returns all share instances with given share_server_id."""
-    return IMPL.share_instances_get_all_by_share_server(
+    return IMPL.share_instance_get_all_by_share_server(
         context, share_server_id, with_share_data=with_share_data)
 
 
-def share_instances_get_all_by_host(context, host, with_share_data=False,
-                                    status=None):
+def share_instance_get_all_by_host(
+    context, host, with_share_data=False, status=None,
+):
     """Returns all share instances with given host."""
-    return IMPL.share_instances_get_all_by_host(
+    return IMPL.share_instance_get_all_by_host(
         context, host, with_share_data=with_share_data, status=status)
 
 
-def share_instances_get_all_by_share_network(context, share_network_id):
+def share_instance_get_all_by_share_network(context, share_network_id):
     """Returns list of shares that belong to given share network."""
-    return IMPL.share_instances_get_all_by_share_network(context,
-                                                         share_network_id)
+    return IMPL.share_instance_get_all_by_share_network(
+        context, share_network_id)
 
 
-def share_instances_get_all_by_share(context, share_id):
+def share_instance_get_all_by_share(context, share_id):
     """Returns list of shares that belong to given share."""
-    return IMPL.share_instances_get_all_by_share(context, share_id)
+    return IMPL.share_instance_get_all_by_share(context, share_id)
 
 
-def share_instances_get_all_by_share_group_id(context, share_group_id):
+def share_instance_get_all_by_share_group_id(context, share_group_id):
     """Returns list of share instances that belong to given share group."""
-    return IMPL.share_instances_get_all_by_share_group_id(
+    return IMPL.share_instance_get_all_by_share_group_id(
         context, share_group_id)
 
 ###################
@@ -451,11 +461,10 @@ def share_get_all_by_share_server(context, share_server_id, filters=None,
         sort_dir=sort_dir)
 
 
-def get_shares_in_recycle_bin_by_share_server(
-        context, share_server_id, filters=None,
-        sort_key=None, sort_dir=None):
+def share_get_all_soft_deleted(
+        context, share_server_id, filters=None, sort_key=None, sort_dir=None):
     """Returns all shares in recycle bin with given share server ID."""
-    return IMPL.get_shares_in_recycle_bin_by_share_server(
+    return IMPL.share_get_all_soft_deleted(
         context, share_server_id, filters=filters, sort_key=sort_key,
         sort_dir=sort_dir)
 
@@ -468,11 +477,10 @@ def share_get_all_by_share_server_with_count(
         sort_dir=sort_dir)
 
 
-def get_shares_in_recycle_bin_by_network(
-        context, share_network_id, filters=None,
-        sort_key=None, sort_dir=None):
+def share_get_all_soft_deleted_by_network(
+        context, share_network_id, filters=None, sort_key=None, sort_dir=None):
     """Returns all shares in recycle bin with given share network ID."""
-    return IMPL.get_shares_in_recycle_bin_by_network(
+    return IMPL.share_get_all_soft_deleted_by_network(
         context, share_network_id, filters=filters, sort_key=sort_key,
         sort_dir=sort_dir)
 
@@ -494,9 +502,9 @@ def share_restore(context, share_id):
 ###################
 
 
-def share_transfer_get(context, transfer_id):
+def transfer_get(context, transfer_id):
     """Get a share transfer record or raise if it does not exist."""
-    return IMPL.share_transfer_get(context, transfer_id)
+    return IMPL.transfer_get(context, transfer_id)
 
 
 def transfer_get_all(context, limit=None, sort_key=None,
@@ -517,6 +525,11 @@ def transfer_get_all_by_project(context, project_id,
                                             filters=filters, offset=offset)
 
 
+def transfer_get_all_expired(context):
+    """Get all expired transfers DB records."""
+    return IMPL.transfer_get_all_expired(context)
+
+
 def transfer_create(context, values):
     """Create an entry in the transfers table."""
     return IMPL.transfer_create(context, values)
@@ -556,6 +569,11 @@ def share_access_get(context, access_id):
     return IMPL.share_access_get(context, access_id)
 
 
+def share_access_get_with_context(context, access_id):
+    """Get share access rule."""
+    return IMPL.share_access_get_with_context(context, access_id)
+
+
 def share_access_get_all_for_share(context, share_id, filters=None):
     """Get all access rules for given share."""
     return IMPL.share_access_get_all_for_share(context, share_id,
@@ -673,9 +691,10 @@ def share_snapshot_instance_delete(context, snapshot_instance_id):
 ####################
 
 
-def share_snapshot_create(context, values):
+def share_snapshot_create(context, values, create_snapshot_instance=True):
     """Create a snapshot from the values dictionary."""
-    return IMPL.share_snapshot_create(context, values)
+    return IMPL.share_snapshot_create(
+        context, values, create_snapshot_instance=create_snapshot_instance)
 
 
 def share_snapshot_get(context, snapshot_id, project_only=True):
@@ -692,6 +711,15 @@ def share_snapshot_get_all(context, filters=None, limit=None, offset=None,
         sort_key=sort_key, sort_dir=sort_dir)
 
 
+def share_snapshot_get_all_with_count(context, filters=None, limit=None,
+                                      offset=None, sort_key=None,
+                                      sort_dir=None):
+    """Get all snapshots."""
+    return IMPL.share_snapshot_get_all_with_count(
+        context, filters=filters, limit=limit, offset=offset,
+        sort_key=sort_key, sort_dir=sort_dir)
+
+
 def share_snapshot_get_all_by_project(context, project_id, filters=None,
                                       limit=None, offset=None, sort_key=None,
                                       sort_dir=None):
@@ -701,6 +729,16 @@ def share_snapshot_get_all_by_project(context, project_id, filters=None,
         sort_key=sort_key, sort_dir=sort_dir)
 
 
+def share_snapshot_get_all_by_project_with_count(context, project_id,
+                                                 filters=None, limit=None,
+                                                 offset=None, sort_key=None,
+                                                 sort_dir=None):
+    """Get all snapshots belonging to a project."""
+    return IMPL.share_snapshot_get_all_by_project_with_count(
+        context, project_id, filters=filters, limit=limit, offset=offset,
+        sort_key=sort_key, sort_dir=sort_dir)
+
+
 def share_snapshot_get_all_for_share(context, share_id, filters=None,
                                      sort_key=None, sort_dir=None):
     """Get all snapshots for a share."""
@@ -735,10 +773,11 @@ def share_snapshot_access_get(context, access_id):
 
 
 def share_snapshot_access_get_all_for_snapshot_instance(
-        context, snapshot_instance_id, session=None):
+    context, snapshot_instance_id,
+):
     """Get all access rules related to a certain snapshot instance."""
     return IMPL.share_snapshot_access_get_all_for_snapshot_instance(
-        context, snapshot_instance_id, session)
+        context, snapshot_instance_id)
 
 
 def share_snapshot_access_get_all_for_share_snapshot(context,
@@ -914,65 +953,67 @@ def share_metadata_update_item(context, share_id, item):
 
 ###################
 
-def share_export_location_get_by_uuid(context, export_location_uuid,
-                                      ignore_secondary_replicas=False):
+def export_location_get_by_uuid(
+    context, export_location_uuid, ignore_secondary_replicas=False,
+):
     """Get specific export location of a share."""
-    return IMPL.share_export_location_get_by_uuid(
+    return IMPL.export_location_get_by_uuid(
         context, export_location_uuid,
         ignore_secondary_replicas=ignore_secondary_replicas)
 
 
-def share_export_locations_get(context, share_id):
+def export_location_get_all(context, share_id):
     """Get all export locations of a share."""
-    return IMPL.share_export_locations_get(context, share_id)
+    return IMPL.export_location_get_all(context, share_id)
 
 
-def share_export_locations_get_by_share_id(context, share_id,
-                                           include_admin_only=True,
-                                           ignore_migration_destination=False,
-                                           ignore_secondary_replicas=False):
+def export_location_get_all_by_share_id(
+    context, share_id,
+    include_admin_only=True,
+    ignore_migration_destination=False,
+    ignore_secondary_replicas=False,
+):
     """Get all export locations of a share by its ID."""
-    return IMPL.share_export_locations_get_by_share_id(
+    return IMPL.export_location_get_all_by_share_id(
         context, share_id, include_admin_only=include_admin_only,
         ignore_migration_destination=ignore_migration_destination,
         ignore_secondary_replicas=ignore_secondary_replicas)
 
 
-def share_export_locations_get_by_share_instance_id(context,
-                                                    share_instance_id,
-                                                    include_admin_only=True):
+def export_location_get_all_by_share_instance_id(
+    context, share_instance_id, include_admin_only=True,
+):
     """Get all export locations of a share instance by its ID."""
-    return IMPL.share_export_locations_get_by_share_instance_id(
+    return IMPL.export_location_get_all_by_share_instance_id(
         context, share_instance_id, include_admin_only=include_admin_only)
 
 
-def share_export_locations_update(context, share_instance_id, export_locations,
-                                  delete=True):
+def export_locations_update(
+    context, share_instance_id, export_locations, delete=True,
+):
     """Update export locations of a share instance."""
-    return IMPL.share_export_locations_update(
+    return IMPL.export_locations_update(
         context, share_instance_id, export_locations, delete)
 
 
 ####################
 
-def export_location_metadata_get(context, export_location_uuid, session=None):
+def export_location_metadata_get(context, export_location_uuid):
     """Get all metadata of an export location."""
-    return IMPL.export_location_metadata_get(
-        context, export_location_uuid, session=session)
+    return IMPL.export_location_metadata_get(context, export_location_uuid)
 
 
-def export_location_metadata_delete(context, export_location_uuid, keys,
-                                    session=None):
+def export_location_metadata_delete(context, export_location_uuid, keys):
     """Delete metadata of an export location."""
     return IMPL.export_location_metadata_delete(
-        context, export_location_uuid, keys, session=session)
+        context, export_location_uuid, keys)
 
 
 def export_location_metadata_update(context, export_location_uuid, metadata,
-                                    delete, session=None):
+                                    delete):
     """Update metadata of an export location."""
     return IMPL.export_location_metadata_update(
-        context, export_location_uuid, metadata, delete, session=session)
+        context, export_location_uuid, metadata, delete)
 
 ####################
 
@@ -1047,13 +1088,6 @@ def share_network_update_security_service(context, id,
         context, id, current_security_service_id, new_security_service_id)
 
 
-def count_share_networks(context, project_id, user_id=None,
-                         share_type_id=None, session=None):
-    return IMPL.count_share_networks(
-        context, project_id, user_id=user_id, share_type_id=share_type_id,
-        session=session,
-    )
-
 ##################
 
 
@@ -1072,18 +1106,20 @@ def share_network_subnet_update(context, network_subnet_id, values):
     return IMPL.share_network_subnet_update(context, network_subnet_id, values)
 
 
-def share_network_subnet_get(context, network_subnet_id, session=None,
-                             parent_id=None):
+def share_network_subnet_get(context, network_subnet_id, parent_id=None):
     """Get requested share network subnet DB record."""
-    return IMPL.share_network_subnet_get(context, network_subnet_id,
-                                         session=session, parent_id=parent_id)
+    return IMPL.share_network_subnet_get(
+        context,
+        network_subnet_id,
+        parent_id=parent_id,
+    )
 
 
-def share_network_subnet_get_all_with_same_az(context, network_subnet_id,
-                                              session=None):
+def share_network_subnet_get_all_with_same_az(context, network_subnet_id):
     """Get requested az share network subnets DB record."""
     return IMPL.share_network_subnet_get_all_with_same_az(
-        context, network_subnet_id, session=session)
+        context, network_subnet_id,
+    )
 
 
 def share_network_subnet_get_all(context):
@@ -1092,8 +1128,9 @@ def share_network_subnet_get_all(context):
 
 
 def share_network_subnets_get_all_by_availability_zone_id(
-        context, share_network_id, availability_zone_id,
-        fallback_to_default=True):
+    context, share_network_id, availability_zone_id,
+    fallback_to_default=True,
+):
     """Get the share network subnets DB record in a given AZ.
 
     This method returns list of subnets DB record for a given share network id
@@ -1105,19 +1142,22 @@ def share_network_subnets_get_all_by_availability_zone_id(
     """
     return IMPL.share_network_subnets_get_all_by_availability_zone_id(
         context, share_network_id, availability_zone_id,
-        fallback_to_default=fallback_to_default)
+        fallback_to_default=fallback_to_default,
+    )
 
 
 def share_network_subnet_get_default_subnets(context, share_network_id):
     """Get the default share network subnets DB records."""
-    return IMPL.share_network_subnet_get_default_subnets(context,
-                                                         share_network_id)
+    return IMPL.share_network_subnet_get_default_subnets(
+        context, share_network_id,
+    )
 
 
 def share_network_subnet_get_all_by_share_server_id(context, share_server_id):
     """Get the subnets that are being used by the share server."""
     return IMPL.share_network_subnet_get_all_by_share_server_id(
-        context, share_server_id)
+        context, share_server_id,
+    )
 
 ####################
 
@@ -1178,19 +1218,18 @@ def network_allocation_update(context, id, values, read_deleted=None):
                                           read_deleted=read_deleted)
 
 
-def network_allocation_get(context, id, session=None, read_deleted=None):
+def network_allocation_get(context, id, read_deleted=None):
     """Get a network allocation DB record."""
-    return IMPL.network_allocation_get(context, id, session,
-                                       read_deleted=read_deleted)
+    return IMPL.network_allocation_get(context, id, read_deleted=read_deleted)
 
 
-def network_allocations_get_for_share_server(context, share_server_id,
-                                             session=None, label=None,
-                                             subnet_id=None):
+def network_allocations_get_for_share_server(
+    context, share_server_id, label=None, subnet_id=None,
+):
     """Get network allocations for share server."""
     return IMPL.network_allocations_get_for_share_server(
-        context, share_server_id, label=label, session=session,
-        subnet_id=subnet_id)
+        context, share_server_id, label=label, subnet_id=subnet_id,
+    )
 
 
 def network_allocations_get_by_ip_address(context, ip_address):
@@ -1216,31 +1255,34 @@ def share_server_update(context, id, values):
     return IMPL.share_server_update(context, id, values)
 
 
-def share_server_get(context, id, session=None):
+def share_server_get(context, id):
     """Get share server DB record by ID."""
-    return IMPL.share_server_get(context, id, session=session)
+    return IMPL.share_server_get(context, id)
 
 
-def share_server_search_by_identifier(context, identifier, session=None):
+def share_server_search_by_identifier(context, identifier):
     """Search for share servers based on given identifier."""
     return IMPL.share_server_search_by_identifier(
-        context, identifier, session=session)
+        context, identifier,
+    )
 
 
-def share_server_get_all_by_host_and_share_subnet_valid(context, host,
-                                                        share_subnet_id,
-                                                        session=None):
+def share_server_get_all_by_host_and_share_subnet_valid(
+    context, host, share_subnet_id,
+):
     """Get share server DB records by host and share net not error."""
     return IMPL.share_server_get_all_by_host_and_share_subnet_valid(
-        context, host, share_subnet_id, session=session)
+        context, host, share_subnet_id,
+    )
 
 
-def share_server_get_all_by_host_and_share_subnet(context, host,
-                                                  share_subnet_id,
-                                                  session=None):
+def share_server_get_all_by_host_and_share_subnet(
+    context, host, share_subnet_id,
+):
     """Get share server DB records by host and share net."""
     return IMPL.share_server_get_all_by_host_and_share_subnet(
-        context, host, share_subnet_id, session=session)
+        context, host, share_subnet_id,
+    )
 
 
 def share_server_get_all(context):
@@ -1264,14 +1306,9 @@ def share_server_get_all_unused_deletable(context, host, updated_before):
                                                       updated_before)
 
 
-def get_all_expired_shares(context):
+def share_get_all_expired(context):
     """Get all expired share DB records."""
-    return IMPL.get_all_expired_shares(context)
-
-
-def get_all_expired_transfers(context):
-    """Get all expired transfers DB records."""
-    return IMPL.get_all_expired_transfers(context)
+    return IMPL.share_get_all_expired(context)
 
 
 def share_server_backend_details_set(context, share_server_id, server_details):
@@ -1492,16 +1529,17 @@ def count_share_group_snapshots_in_share_group(context, share_group_id):
         context, share_group_id)
 
 
-def count_share_groups_in_share_network(context, share_network_id,
-                                        session=None):
+def count_share_groups_in_share_network(context, share_network_id):
     """Return the number of groups with the specified share network."""
     return IMPL.count_share_groups_in_share_network(context, share_network_id)
 
 
-def count_share_group_snapshot_members_in_share(context, share_id,
-                                                session=None):
+def count_share_group_snapshot_members_in_share(
+    context, share_id, include_deferred_deleting=True
+):
     """Returns the number of group snapshot members linked to the share."""
-    return IMPL.count_share_group_snapshot_members_in_share(context, share_id)
+    return IMPL.count_share_group_snapshot_members_in_share(
+        context, share_id, include_deferred_deleting=include_deferred_deleting)
 
 
 def share_group_snapshot_get(context, share_group_snapshot_id):
@@ -1611,11 +1649,10 @@ def share_replica_update(context, share_replica_id, values,
                                      with_share_data=with_share_data)
 
 
-def share_replica_delete(context, share_replica_id,
-                         need_to_update_usages=True):
+def share_replica_delete(context, replica_id, need_to_update_usages=True):
     """Deletes a share replica."""
     return IMPL.share_replica_delete(
-        context, share_replica_id, need_to_update_usages=need_to_update_usages)
+        context, replica_id, need_to_update_usages=need_to_update_usages)
 
 
 def purge_deleted_records(context, age_in_days):
@@ -1771,3 +1808,60 @@ def async_operation_data_update(context, entity_id, details,
 def async_operation_data_delete(context, entity_id, key=None):
     """Remove one, list or all key-value pairs for given entity_id."""
     return IMPL.async_operation_data_delete(context, entity_id, key)
+
+####################
+
+
+def share_backup_create(context, share_id, values):
+    """Create new share backup with specified values."""
+    return IMPL.share_backup_create(context, share_id, values)
+
+
+def share_backup_update(context, backup_id, values):
+    """Updates a share backup with given values."""
+    return IMPL.share_backup_update(context, backup_id, values)
+
+
+def share_backup_get(context, backup_id):
+    """Get share backup by id."""
+    return IMPL.share_backup_get(context, backup_id)
+
+
+def share_backups_get_all(context, filters=None, limit=None, offset=None,
+                          sort_key=None, sort_dir=None):
+    """Get all backups."""
+    return IMPL.share_backups_get_all(
+        context, filters=filters, limit=limit, offset=offset,
+        sort_key=sort_key, sort_dir=sort_dir)
+
+
+def share_backup_delete(context, backup_id):
+    """Deletes backup with the specified ID."""
+    return IMPL.share_backup_delete(context, backup_id)
+
+#####################
+
+
+def resource_lock_create(context, values):
+    """Create a resource lock."""
+    return IMPL.resource_lock_create(context, values)
+
+
+def resource_lock_update(context, lock_id, values):
+    """Update a resource lock."""
+    return IMPL.resource_lock_update(context, lock_id, values)
+
+
+def resource_lock_delete(context, lock_id):
+    """Delete a resource lock."""
+    return IMPL.resource_lock_delete(context, lock_id)
+
+
+def resource_lock_get(context, lock_id):
+    """Retrieve a resource lock."""
+    return IMPL.resource_lock_get(context, lock_id)
+
+
+def resource_lock_get_all(context, **kwargs):
+    """Retrieve all resource locks."""
+    return IMPL.resource_lock_get_all(context, **kwargs)
