@@ -30,6 +30,7 @@ from oslo_utils import netutils
 from manila.common import constants as const
 from manila import compute
 from manila import context
+from manila import coordination
 from manila import exception
 from manila.i18n import _
 from manila import image
@@ -818,7 +819,7 @@ class NeutronNetworkHelper(BaseNetworkhelper):
         net = self.neutron_api.get_network(network_info['neutron_net_id'])
         return net['name']
 
-    @utils.synchronized("service_instance_get_service_network", external=True)
+    @coordination.synchronized("service_instance_get_service_network")
     def _get_service_network_id(self):
         """Finds existing or creates new service network."""
         service_network_name = self.get_config_option("service_network_name")
@@ -855,7 +856,7 @@ class NeutronNetworkHelper(BaseNetworkhelper):
                     LOG.debug("Failed to delete port %(port_id)s with error: "
                               "\n %(exc)s", {"port_id": port_id, "exc": e})
 
-        if router_id and subnet_id:
+        if subnet_id:
             ports = self.neutron_api.list_ports(
                 fields=['device_id', 'device_owner'],
                 fixed_ips=['subnet_id=%s' % subnet_id])
@@ -872,17 +873,18 @@ class NeutronNetworkHelper(BaseNetworkhelper):
                     # exist that use this subnet. So, do not remove it
                     # from router.
                     return
-            try:
-                # NOTE(vponomaryov): there is no other share servers or
-                # some VMs that use this subnet. So, remove it from router.
-                self.neutron_api.router_remove_interface(
-                    router_id, subnet_id)
-            except exception.NetworkException as e:
-                if e.kwargs['code'] != 404:
-                    raise
-                LOG.debug('Subnet %(subnet_id)s is not attached to the '
-                          'router %(router_id)s.',
-                          {'subnet_id': subnet_id, 'router_id': router_id})
+            if router_id:
+                try:
+                    # NOTE(vponomaryov): there is no other share servers or
+                    # some VMs that use this subnet. So, remove it from router.
+                    self.neutron_api.router_remove_interface(
+                        router_id, subnet_id)
+                except exception.NetworkException as e:
+                    if e.kwargs['code'] != 404:
+                        raise
+                    LOG.debug('Subnet %(subnet_id)s is not attached to the '
+                              'router %(router_id)s.',
+                              {'subnet_id': subnet_id, 'router_id': router_id})
             self.neutron_api.update_subnet(subnet_id, '')
 
     @utils.synchronized(
